@@ -1,9 +1,10 @@
 import random
+import sys
 from time import sleep
 import asyncio
+import os
 
-
-# from python_redis_orm.core import *
+from python_redis_orm.core import *
 
 
 def generate_token(chars_count):
@@ -56,6 +57,28 @@ class ListCheckModel(RedisModel):
     redis_list = RedisList()
 
 
+class ForeignKeyCheckModel(RedisModel):
+    task_challenge = RedisForeignKey(model=TaskChallenge)
+
+
+class ManyToManyCheckModel(RedisModel):
+    task_challenges = RedisManyToMany(model=TaskChallenge)
+
+
+class ModelWithOverriddenSave(RedisModel):
+    multiplied_max_field = RedisNumber()
+    
+    def save(self):
+        redis_root = self.get('redis_root')  # get value of any field
+        new_value = 1
+        all_instances = redis_root.get(ModelWithOverriddenSave)
+        if all_instances:
+            max_value = max(list(map(lambda instance: instance['multiplied_max_field'], all_instances)))
+            new_value = max_value * 2
+        self.set(multiplied_max_field=new_value)
+        return super().save()
+
+
 def clean_db_after_test(connection_pool, prefix):
     redis_instance = redis.Redis(connection_pool=connection_pool)
     for key in redis_instance.keys(f'{prefix}:*'):
@@ -128,8 +151,8 @@ def no_connection_pool_test(*args, **kwargs):
         task_challenges = redis_root.get(TaskChallenge)
         have_exception = False
         connection_pool = redis.ConnectionPool(
-            host='localhost',
-            port=6379,
+            host=os.environ['REDIS_HOST'],
+            port=os.environ['REDIS_PORT'],
             db=0,
             decode_responses=True
         )
@@ -519,7 +542,12 @@ def async_test(connection_pool, prefix):
         
         async def async_task(data_count, use_async=True):
             redis_roots = []
-            connection_pool = redis.ConnectionPool(host='localhost', port=6379, db=0, decode_responses=True)
+            connection_pool = redis.ConnectionPool(
+                host=os.environ['REDIS_HOST'],
+                port=os.environ['REDIS_PORT'],
+                db=0,
+                decode_responses=True
+            )
             for i in range(data_count):
                 redis_root = RedisRoot(
                     prefix=prefix,
@@ -664,10 +692,104 @@ def async_test(connection_pool, prefix):
     return have_exception
 
 
+def foreign_key_test(connection_pool, prefix):
+    redis_root = RedisRoot(
+        prefix=prefix,
+        connection_pool=connection_pool,
+        ignore_deserialization_errors=True,
+    )
+    have_exception = False
+    try:
+        task_id = 12345
+        task_challenge = TaskChallenge(
+            redis_root=redis_root,
+            task_id=task_id
+        ).save()
+        foreign_key_check_instance = redis_root.create(
+            ForeignKeyCheckModel,
+            task_challenge=task_challenge
+        )
+        # Check really created
+        task_challenge_qs = redis_root.get(TaskChallenge, task_id=task_id)
+        if len(task_challenge_qs) != 1:
+            have_exception = True
+        else:
+            task_challenge = task_challenge_qs[0]
+            foreign_key_check_instance_qs = redis_root.get(ForeignKeyCheckModel, task_challenge=task_challenge)
+            if len(foreign_key_check_instance_qs) != 1:
+                have_exception = True
+            else:
+                foreign_key_check_instance = foreign_key_check_instance_qs[0]
+                if foreign_key_check_instance['task_challenge']['task_id'] != task_id:
+                    have_exception = True
+    except BaseException as ex:
+        print(ex)
+        have_exception = True
+    
+    clean_db_after_test(connection_pool, prefix)
+    return have_exception
+
+
+def many_to_many_test(connection_pool, prefix):
+    redis_root = RedisRoot(
+        prefix=prefix,
+        connection_pool=connection_pool,
+        ignore_deserialization_errors=True,
+    )
+    have_exception = False
+    try:
+        tasks_ids = set([random.randrange(0, 100) for i in range(10)])
+        task_challenges = [
+            TaskChallenge(
+                redis_root=redis_root,
+                task_id=task_id
+            ).save()
+            for task_id in tasks_ids
+        ]
+        many_to_many_check_instance = redis_root.create(
+            ManyToManyCheckModel,
+            task_challenges=task_challenges
+        )
+        # Check really created
+        many_to_many_check_instances_qs = redis_root.get(ManyToManyCheckModel)
+        if len(many_to_many_check_instances_qs) != 1:
+            have_exception = True
+        else:
+            many_to_many_check_instance = many_to_many_check_instances_qs[0]
+            if many_to_many_check_instance['task_challenges'] != task_challenges:
+                have_exception = True
+    except BaseException as ex:
+        print(ex)
+        have_exception = True
+    
+    clean_db_after_test(connection_pool, prefix)
+    return have_exception
+
+
+def save_override_test(connection_pool, prefix):
+    redis_root = RedisRoot(
+        prefix=prefix,
+        connection_pool=connection_pool,
+        ignore_deserialization_errors=True,
+    )
+    have_exception = False
+    try:
+        instance_1 = redis_root.create(ModelWithOverriddenSave)
+        instance_2 = redis_root.create(ModelWithOverriddenSave)
+        if instance_1['multiplied_max_field'] * 2 != instance_2['multiplied_max_field']:
+            have_exception = True
+    except BaseException as ex:
+        print(ex)
+        have_exception = True
+    
+    clean_db_after_test(connection_pool, prefix)
+    return have_exception
+
+
 def run_tests():
     connection_pool = redis.ConnectionPool(
-        host='localhost',
-        port=6379,
+        host=os.environ['REDIS_HOST'],
+        port=os.environ['REDIS_PORT'],
         db=0,
         decode_responses=True
     )
@@ -689,6 +811,9 @@ def run_tests():
         list_test,
         dict_test,
         async_test,
+        foreign_key_test,
+        many_to_many_test,
+        save_override_test,
     ]
     results = []
     started_in = datetime.datetime.now()
@@ -715,7 +840,11 @@ def run_tests():
     print(f'\n'
           f'{results_success_count} / {len(results)} tests ran successfully\n'
           f'All tests completed in {time}s\n')
+    
+    return all(results)
 
 
 if __name__ == '__main__':
-    run_tests()
+    results = run_tests()
+    if not results:
+        sys.exit(1)
